<?php

namespace Drupal\commerce_globalonepay\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OnsitePaymentGatewayInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsAuthorizationsInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsRefundsInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use SimpleXMLElement;

/**
 * Provides the interface for the commerce_globalonepay payment gateway.
 */
interface GlobalOnePayOnsiteInterface extends OnsitePaymentGatewayInterface, SupportsAuthorizationsInterface, SupportsRefundsInterface {

  /**
   * Returns the Api URL.
   */
  public function getApiUrl();

  /**
   * Returns the Terminal ID.
   */
  public function getTerminalId();

  /**
   * Returns the Shared Secret.
   */
  public function getSharedSecret();

  /**
   * Generates unique merchant reference number.
   *
   * It contains User ID + current time + Unique ID with random prefix value.
   */
  public function generateMerchantRef();

  /**
   * Generates MD5 HASH based on provided values.
   *
   * @param array $parameters
   *   Parameters array needed to build the HASH.
   *
   * @return string
   *   String with the hash as a 32-character hexadecimal number.
   */
  public function generateHash(array $parameters);

  /**
   * Format the expiration date for GlobalOne from the provided payment details.
   *
   * @param array $payment_details
   *   The payment details array.
   *
   * @return string
   *   The expiration date string.
   */
  public function getExpirationDate(array $payment_details);

  /**
   * Returns unique Order ID.
   *
   * We need to return unique Order ID. In live mode this will be an Order ID.
   * In test mode - Order ID + current time (so it will be unique on different)
   * environments (local/dev/qa).
   *
   * @param \Drupal\commerce_payment\Entity\PaymentInterface $payment
   *   The payment.
   */
  public function getOrderId(PaymentInterface $payment);

  /**
   * Prepares the request body to name/value pairs.
   *
   * @param array $parameters
   *   The request parameters.
   * @param \SimpleXMLElement $xml
   *   New XML object.
   *
   * @return string
   *   The request body.
   */
  public function prepareBody(array $parameters, SimpleXMLElement &$xml);

  /**
   * Convert the result of a request from object to array.
   *
   * @param string $xml_string
   *   The result.
   *
   * @return array
   *   An array of the result values.
   */
  public function prepareResult($xml_string);

  /**
   * Post a transaction to the GlobalOnePay server and return the response.
   *
   * @param string $type
   *   Type of current operation.
   * @param array $parameters
   *   The parameters to send.
   *
   * @return array
   *   The response body data in array format.
   */
  public function executeTransaction($type, array $parameters);

}
